/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl;

import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.wordsource.WordSourceId;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.UnknownNamespaceException;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import java.util.Locale;


/**
 * <p>
 * Defines utilities for persistence classes.
 * </p>
 *
 * <p>
 * This class that provides some utility methods to simplify argument checking and create objects using reflection.
 * </p>
 *
 * <p>
 * This class is made as public since it will be accessed by multi-packages.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public final class DocumentIndexerPersistenceHelper {
    /** Represents the hexadecimal string used to convert a byte to be hexadecimal. */
    private static String[] hexDigits = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e",
        "f"};

    /**
     * Private constructor to prevent this class be instantiated.
     */
    private DocumentIndexerPersistenceHelper() {
        // empty
    }

    /**
     * <p>
     * Generates the string representation of the given <code>WordSourceId</code> instance. This value will be used as
     * an identifier of the document index.
     * </p>
     *
     * @param wordSourceId the given <code>WordSourceId</code> instance to be represented as a string value.
     *
     * @return the string representation of the given <code>WordSourceId</code> instance
     *
     * @throws IndexPersistenceException if it fails to generate the string representation of the given
     *         <code>WordSourceId</code> instance.
     */
    public static String getDocumentId(WordSourceId wordSourceId) throws IndexPersistenceException {
        // create a serialized stream
        byte[] sourceIdentity = getSerializedBytes(wordSourceId.getSourceIdentity());
        byte[] className = getSerializedBytes(wordSourceId.getSourceClassName());
        byte[] locale = getSerializedBytes(wordSourceId.getSourceLocale());
        byte[] values = getSerializedBytes(wordSourceId.getDelimiters());
        byte[] wordSourceIdBytes = new byte[sourceIdentity.length + className.length + locale.length + values.length];
        System.arraycopy(sourceIdentity, 0, wordSourceIdBytes, 0, sourceIdentity.length);
        System.arraycopy(className, 0, wordSourceIdBytes, sourceIdentity.length, className.length);
        System.arraycopy(locale, 0, wordSourceIdBytes, sourceIdentity.length + className.length, locale.length);
        System.arraycopy(values, 0, wordSourceIdBytes, sourceIdentity.length + className.length + locale.length,
            values.length);

        // use MD5 (with 32 char hex encoding as the final format) to generate the hash from the data in the stream.
        return getDigestBasedId(new ByteArrayInputStream(wordSourceIdBytes));
    }

    /**
     * <p>
     * Generates the string representation using the bytes retrieved from the given input stream.
     * </p>
     *
     * @param is the input stream where to retrieve the bytes.
     *
     * @return the string representation using the bytes retrieved from the given input stream.
     *
     * @throws IndexPersistenceException if it fails to generate the string representation.
     */
    private static String getDigestBasedId(InputStream is) throws IndexPersistenceException {
        DigestInputStream in = null;
        final int bufferSize = 1024;

        try {
            // Create an MD5 message digest
            MessageDigest msgDigest = MessageDigest.getInstance("MD5");
            in = new DigestInputStream(is, msgDigest);

            int n;
            byte[] buffer = new byte[bufferSize]; // BUFFER_SIZE

            do {
                n = in.read(buffer, 0, bufferSize);
            } while (n == bufferSize);

            // Get the final digest and convert it to a String.
            msgDigest = in.getMessageDigest();

            byte[] digest = msgDigest.digest();
            StringBuffer stringBuffer = new StringBuffer();

            for (int i = 0; i < digest.length; i++) {
                // Encode it into 32 char hex encoding using lower case letters.
                stringBuffer.append(byteToHexString(digest[i]));
            }

            // Return the digest as a String.
            return stringBuffer.toString();
        } catch (NoSuchAlgorithmException e) {
            throw new IndexPersistenceException("No such algorithm exists.", e);
        } catch (IOException e) {
            throw new IndexPersistenceException("IO errors happen.", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    // ignore
                }
            }
        }
    }

    /**
     * <p>
     * Converts a byte to a hexadecimal String.
     * </p>
     *
     * @param b the byte to convert.
     *
     * @return the hexadecimal String.
     */
    private static String byteToHexString(byte b) {
        int num = b;

        if (num < 0) {
            num = 256 + num;
        }

        int digit1 = num / 16;
        int digit2 = num % 16;

        return hexDigits[digit1] + hexDigits[digit2];
    }

    /**
     * <p>
     * Gets the serialized bytes from the given object.
     * </p>
     *
     * @param object the given object to get the serialized bytes.
     *
     * @return the serialized bytes from the given object.
     *
     * @throws IndexPersistenceException if it fails to get the serialized bytes from the given object or the given
     *         object is not a sub-class of <code>Serializable</code>.
     */
    public static byte[] getSerializedBytes(Object object) throws IndexPersistenceException {
        if (!(object instanceof Serializable)) {
            throw new IndexPersistenceException("The object is not serializable.");
        }

        try {
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
            objectOutputStream.writeObject(object);

            return byteArrayOutputStream.toByteArray();
        } catch (IOException e) {
            throw new IndexPersistenceException("Error happens in the writing serialized bytes process.", e);
        }
    }

    /**
     * <p>
     * Gets the object from the serialized bytes.
     * </p>
     *
     * @param bytes the serialized bytes to get the object.
     * @param offset the offset in the buffer of the first byte to read.
     * @param length the maximum number of bytes to read from the buffer.
     *
     * @return the object from the serialized bytes.
     *
     * @throws IndexPersistenceException if it fails to get the object from the serialized bytes.
     */
    public static Object getSerializedObject(byte[] bytes, int offset, int length) throws IndexPersistenceException {
        try {
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(bytes, offset, length);
            ObjectInputStream objectInputStream = new ObjectInputStream(byteArrayInputStream);

            return objectInputStream.readObject();
        } catch (IOException e) {
            throw new IndexPersistenceException("Error happens in the writing serialized bytes process.", e);
        } catch (ClassNotFoundException e) {
            throw new IndexPersistenceException("Error happens to the retrieve the expected class.", e);
        }
    }

    /**
     * <p>
     * Get the string value in <code>ConfigManager</code> with specified namespace and name of property.
     * </p>
     *
     * @param namespace the namespace of the property.
     * @param name the name of the property.
     * @param required whether the property is required.
     *
     * @return the string value in <code>ConfigManager</code> with specified namespace and name of property.
     *
     * @throws PersistenceConfigurationException if the namespace doesn't exist or the property doesn't exist when it
     *         is required.
     */
    public static String getStringProperty(String namespace, String name, boolean required)
        throws PersistenceConfigurationException {
        try {
            String value = (String) ConfigManager.getInstance().getString(namespace, name);

            // if the value is required and doesn't exist, throw exception.
            if (required && (value == null)) {
                throw new PersistenceConfigurationException("The required parameter " + name + " in namespace "
                    + namespace + " doesn't exist.");
            }

            return value;
        } catch (UnknownNamespaceException une) {
            throw new PersistenceConfigurationException("The namespace with the name of " + namespace
                + " doesn't exist.", une);
        }
    }

    /**
     * <p>
     * Parses the given locale string into the <code>Locale</code> instance. The locale string may be in the following
     * format: Language_Country_Version, Language_Country, Language.
     * </p>
     *
     * @param localeString the given locale string to parse into the <code>Locale</code> instance.
     *
     * @return the parsed <code>Locale</code> instance.
     *
     * @throws IndexPersistenceException if the locale string is not well formatted.
     */
    public static Locale parseLocaleString(String localeString) throws IndexPersistenceException {
        String[] parts = localeString.split("_");

        if (parts.length == 3) {
            // the format should be like this: Language_Country_Version
            return new Locale(parts[0], parts[1], parts[2]);
        } else if (parts.length == 2) {
            // the format should be like this: Language_Country
            return new Locale(parts[0], parts[1]);
        } else if (parts.length == 1) {
            // the format should be like this: Language
            return new Locale(parts[0]);
        }

        throw new IndexPersistenceException("The locale string is not well formatted.");
    }

    /**
     * <p>
     * Get the int value in <code>ConfigManager</code> with specified namespace and name of property.
     * </p>
     *
     * @param namespace the namespace of the property.
     * @param name the name of the property.
     *
     * @return the int value in <code>ConfigManager</code> with specified namespace and name of property.
     *
     * @throws PersistenceConfigurationException if the namespace doesn't exist or the property doesn't exist or the
     *         perperty value is incorrect.
     */
    public static int getIntProperty(String namespace, String name) throws PersistenceConfigurationException {
        String stringValue = getStringProperty(namespace, name, true);

        try {
            return Integer.parseInt(stringValue);
        } catch (NumberFormatException e) {
            throw new PersistenceConfigurationException("The int value for property " + name
                + " is not well formatted.", e);
        }
    }

    /**
     * <p>
     * Converts the string value. The special characters will be converted to other representations which will be
     * successfully parsed in the xml file.
     * </p>
     *
     * @param value the string value to be converted.
     *
     * @return the converted string value.
     */
    public static String convertString(String value) {
        // convert the special characters
        StringBuffer buffer = new StringBuffer();

        for (int i = 0; i < value.length(); i++) {
            if ((value.charAt(i) < 32) || (value.charAt(i) >= 127)) {
                buffer.append("&#").append((int) value.charAt(i)).append(";");
            } else if (value.charAt(i) == '"') {
                buffer.append("&quot;");
            } else if (value.charAt(i) == '<') {
                buffer.append("&lt;");
            } else if (value.charAt(i) == '>') {
                buffer.append("&gt;");
            } else if (value.charAt(i) == '\'') {
                buffer.append("&apos;");
            } else if (value.charAt(i) == '&') {
                buffer.append("&amp;");
            } else {
                buffer.append(value.charAt(i));
            }
        }

        return buffer.toString();
    }

    /**
     * <p>
     * Validates the value of a variable with type <code>Object</code>. The value cannot be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is <code>null</code>.
     */
    public static void validateNotNull(Object value, String name) {
        if (value == null) {
            throw new IllegalArgumentException(name + " cannot be null.");
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be empty string, but may be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is empty string.
     */
    public static void validateNotEmpty(String value, String name) {
        if (value != null) {
            if (value.trim().length() == 0) {
                throw new IllegalArgumentException(name + " cannot be empty string.");
            }
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be <code>null</code> or empty string.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is <code>null</code> or is empty string.
     */
    public static void validateString(String value, String name) {
        validateNotNull(value, name);
        validateNotEmpty(value, name);
    }
}
