/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;


/**
 * <p>
 * This class contains methods with SQL update commands used by <code>FastAccessDBIndexPersistence</code> and
 * <code>SimpleAccessDBIndexPersistence</code>.
 * </p>
 *
 * <p>
 * Each method accepts a connection argument, and usually represents single sql statement.
 * </p>
 *
 * <p>
 * This class is for single-threaded use and should be disposed after use (to release prepared statements)
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
final class SQLUpdates {
    /** Represents the prepared SQL statement to add a record to WORD_DATA table. */
    private static final String CREATE_WORD_DATA_SQL = "INSERT INTO WORD_DATA (DOCUMENT_ID, WORD_DATA) VALUES (?, ?)";

    /** Represents the prepared SQL statement to add a record to DOCUMENT_COLLECTION table. */
    private static final String CREATE_DOCUMENT_COLLECTION_SQL = "INSERT INTO "
        + "DOCUMENT_COLLECTION (COLLECTION_ID) VALUES (?)";

    /** Represents the prepared SQL statement to add a record to CREATE_DOCUMENT_COLLECTION_DOCUMENT_XREF_SQL table. */
    private static final String CREATE_DOCUMENT_COLLECTION_DOCUMENT_XREF_SQL = "INSERT INTO "
        + "DOCUMENT_COLLECTION_DOCUMENT_XREF (COLLECTION_ID, DOCUMENT_ID) VALUES (?,?)";

    /** Represents the prepared SQL statement to add a record to DOCUMENT table. */
    private static final String CREATE_DOCUMENT_SQL = "INSERT INTO "
        + "DOCUMENT (DOCUMENT_ID, LOCALE, SOURCE_IDENTITY, CLASS_NAME, USE_COUNT) VALUES (?,?,?,?,?)";

    /** Represents the prepared SQL statement to add a record to DELIMITER table. */
    private static final String CREATE_DELIMITER_SQL = "INSERT INTO "
        + "DELIMITER (DOCUMENT_ID, DELIMITER_ENTRY, DELIMITER_SORT_ORDER) VALUES (?,?,?)";

    /** Represents the prepared SQL statement to add a record to WORD table. */
    private static final String CREATE_WORD_SQL = "INSERT INTO WORD (DOCUMENT_ID, WORD_ID, WORD_ENTRY) VALUES (?,?,?)";

    /** Represents the prepared SQL statement to add a record to WORD_POSITION table. */
    private static final String CREATE_WORD_POSITION_SQL = "INSERT INTO "
        + "WORD_POSITION (DOCUMENT_ID, WORD_ID, WORD_POSITION) VALUES (?,?,?)";

    /** Represents the prepared SQL statement to update a record in DOCUMENT table. */
    private static final String UPDATE_DOCUMENT_USE_COUNT_SQL = "UPDATE DOCUMENT SET USE_COUNT = ? "
        + "WHERE DOCUMENT_ID = ?";

    /**
     * Represents the prepared SQL statement to delete a record from DOCUMENT_COLLECTION table with given collection
     * id.
     */
    private static final String DELETE_DOCUMENT_COLLECTION_COLLECTIONID_SQL = "DELETE FROM "
        + "DOCUMENT_COLLECTION WHERE COLLECTION_ID = ?";

    /**
     * Represents the prepared SQL statement to delete a record from DOCUMENT_COLLECTION_DOCUMENT_XREF table with given
     * collection id.
     */
    private static final String DELETE_DOCUMENT_COLLECTION_DOCUMENT_XREF_COLLECTIONID_SQL = "DELETE "
        + "FROM DOCUMENT_COLLECTION_DOCUMENT_XREF WHERE COLLECTION_ID = ?";

    /**
     * Represents the prepared SQL statement to delete a record from DOCUMENT_COLLECTION_DOCUMENT_XREF table with given
     * collection id and document id.
     */
    private static final String DELETE_DOCUMENT_COLLECTION_DOCUMENT_XREF_COLLECTIONID_DOCUMENTID_SQL = "DELETE "
        + "FROM DOCUMENT_COLLECTION_DOCUMENT_XREF WHERE COLLECTION_ID = ? AND DOCUMENT_ID = ?";

    /** Represents the prepared SQL statement to delete a record from DOCUMENT table with given document id. */
    private static final String DELETE_DOCUMENT_DOCUMENTID_SQL = "DELETE FROM DOCUMENT WHERE DOCUMENT_ID = ?";

    /** Represents the prepared SQL statement to delete a record from DELIMITER table with given document id. */
    private static final String DELETE_DELIMITER_DOCUMENTID_SQL = "DELETE FROM DELIMITER WHERE DOCUMENT_ID = ?";

    /** Represents the prepared SQL statement to delete a record from WORD table with given document id. */
    private static final String DELETE_WORD_DOCUMENTID_SQL = "DELETE FROM WORD WHERE DOCUMENT_ID = ?";

    /** Represents the prepared SQL statement to delete a record from WORD_POSITION table with given document id. */
    private static final String DELETE_WORD_POSITION_DOCUMENTID_SQL = "DELETE FROM WORD_POSITION WHERE DOCUMENT_ID = ?";

    /** Represents the prepared SQL statement to delete a record from WORD_DATA table with given document id. */
    private static final String DELETE_WORD_DATA_DOCUMENTID_SQL = "DELETE FROM WORD_DATA WHERE DOCUMENT_ID = ?";

    /**
     * Private constructor to prevent this class be instantiated.
     */
    private SQLUpdates() {
        // empty
    }

    /**
     * <p>
     * Adds a record to DOCUMENT table.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id of the document.
     * @param locale the locale of the document.
     * @param sourceIdentity the sourceIdentity of the document.
     * @param className the className of the document.
     * @param useCount the useCount of the document.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int insertDocumentTable(Connection connection, String documentId, String locale,
        byte[] sourceIdentity, String className, int useCount) throws SQLException {
        return excuteUpdates(connection, CREATE_DOCUMENT_SQL,
            new Object[] {documentId, locale, sourceIdentity, className, new Integer(useCount)});
    }

    /**
     * <p>
     * Adds a record to WORD_DATA table.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id of the word data.
     * @param wordEntry the wordEntry of the word data.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int insertWordDataTable(Connection connection, String documentId, String wordEntry)
        throws SQLException {
        return excuteUpdates(connection, CREATE_WORD_DATA_SQL, new Object[] {documentId, wordEntry});
    }

    /**
     * <p>
     * Adds a record to DELIMITER table.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id of the delimiter.
     * @param delimiter the delimiter of the delimiter.
     * @param order the delimiter order of the delimiter.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int insertDelimiterTable(Connection connection, String documentId, String delimiter, int order)
        throws SQLException {
        StringBuffer buffer = new StringBuffer();
        // escape the character of '\0'
        for (int i = 0; i < delimiter.length(); i++) {
            if (delimiter.charAt(i) == '\0') {
                buffer.append("\\0");
            } else if (delimiter.charAt(i) == '\\') {
                buffer.append("\\\\");
            } else {
                buffer.append(delimiter.charAt(i));
            }
        }
        return excuteUpdates(connection, CREATE_DELIMITER_SQL,
                new Object[] {documentId, buffer.toString(), new Integer(order)});
    }

    /**
     * <p>
     * Adds a record to WORD table.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id of the word.
     * @param wordId the word id of the word.
     * @param word the word of the word.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int insertWordTable(Connection connection, String documentId, int wordId, String word)
        throws SQLException {
        return excuteUpdates(connection, CREATE_WORD_SQL, new Object[] {documentId, new Integer(wordId), word});
    }

    /**
     * <p>
     * Adds a record to WORD_POSITION table.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id of the word position.
     * @param wordId the word id of the word position.
     * @param position the position of the word position.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int insertWordPositionTable(Connection connection, String documentId, int wordId, Integer position)
        throws SQLException {
        return excuteUpdates(connection, CREATE_WORD_POSITION_SQL,
            new Object[] {documentId, new Integer(wordId), position});
    }

    /**
     * <p>
     * Adds a record to DOCUMENT_COLLECTION table.
     * </p>
     *
     * @param connection the connection to access database.
     * @param collectionId the collection id of the document collection.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int insertDocumentCollectionTable(Connection connection, String collectionId)
        throws SQLException {
        return excuteUpdates(connection, CREATE_DOCUMENT_COLLECTION_SQL, new Object[] {collectionId});
    }

    /**
     * <p>
     * Adds a record to CREATE_DOCUMENT_COLLECTION_DOCUMENT_XREF_SQL table.
     * </p>
     *
     * @param connection the connection to access database.
     * @param collectionId the collection id of the document collection xref.
     * @param documentId the document id of the document collection xref.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int insertDocumentCollectionDocumentXref(Connection connection,
            String collectionId, String documentId)
        throws SQLException {
        return excuteUpdates(connection, CREATE_DOCUMENT_COLLECTION_DOCUMENT_XREF_SQL,
            new Object[] {collectionId, documentId});
    }

    /**
     * <p>
     * Excutes the update operation of the database.
     * </p>
     *
     * @param connection the connection to access database.
     * @param sqlStr the prepared SQL statement string to do the update operation.
     * @param parameters the parameters to set the prepared SQL statement to do the update operation.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    private static int excuteUpdates(Connection connection, String sqlStr, Object[] parameters)
        throws SQLException {
        PreparedStatement statement = null;

        try {
            statement = connection.prepareStatement(sqlStr);

            for (int i = 0; i < parameters.length; i++) {
                if (parameters[i] instanceof String) {
                    statement.setString(i + 1, (String) parameters[i]);
                } else {
                    statement.setObject(i + 1, parameters[i]);
                }
            }

            return statement.executeUpdate();
        } finally {
            closeResources(statement);
        }
    }

    /**
     * <p>
     * Delete a record from DOCUMENT table with given document id.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id to delete.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int deleteDocumentTable(Connection connection, String documentId) throws SQLException {
        return excuteUpdates(connection, DELETE_DOCUMENT_DOCUMENTID_SQL, new Object[] {documentId});
    }

    /**
     * <p>
     * Delete a record from DOCUMENT_COLLECTION table with given collection id.
     * </p>
     *
     * @param connection the connection to access database.
     * @param collectionId the collection id to delete.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int deleteDocumentCollectionTable(Connection connection, String collectionId)
        throws SQLException {
        return excuteUpdates(connection, DELETE_DOCUMENT_COLLECTION_COLLECTIONID_SQL, new Object[] {collectionId});
    }

    /**
     * <p>
     * Delete a record from DOCUMENT_COLLECTION_DOCUMENT_XREF table with given collection id.
     * </p>
     *
     * @param connection the connection to access database.
     * @param collectionId the collection id to delete.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int deleteDocumentCollectionDocumentXrefTable(Connection connection, String collectionId)
        throws SQLException {
        return excuteUpdates(connection, DELETE_DOCUMENT_COLLECTION_DOCUMENT_XREF_COLLECTIONID_SQL,
            new Object[] {collectionId});
    }

    /**
     * <p>
     * Delete a record from DOCUMENT_COLLECTION_DOCUMENT_XREF table with given collection id and document id.
     * </p>
     *
     * @param connection the connection to access database.
     * @param collectionId the collection id to delete.
     * @param documentId the document id to delete.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int deleteDocumentCollectionDocumentXrefTable(Connection connection, String collectionId,
        String documentId) throws SQLException {
        return excuteUpdates(connection, DELETE_DOCUMENT_COLLECTION_DOCUMENT_XREF_COLLECTIONID_DOCUMENTID_SQL,
            new Object[] {collectionId, documentId});
    }

    /**
     * <p>
     * Delete a record from DELIMITER table with given document id.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id to delete.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int deleteDelimiterTable(Connection connection, String documentId) throws SQLException {
        return excuteUpdates(connection, DELETE_DELIMITER_DOCUMENTID_SQL, new Object[] {documentId});
    }

    /**
     * <p>
     * Delete a record from WORD table with given document id.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id to delete.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int deleteWordTable(Connection connection, String documentId) throws SQLException {
        return excuteUpdates(connection, DELETE_WORD_DOCUMENTID_SQL, new Object[] {documentId});
    }

    /**
     * <p>
     * Delete a record from WORD_POSITION table with given document id.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id to delete.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int deleteWordPositionTable(Connection connection, String documentId) throws SQLException {
        return excuteUpdates(connection, DELETE_WORD_POSITION_DOCUMENTID_SQL, new Object[] {documentId});
    }

    /**
     * <p>
     * Delete a record from WORD_DATA table with given document id.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id to delete.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int deleteWordDataTable(Connection connection, String documentId) throws SQLException {
        return excuteUpdates(connection, DELETE_WORD_DATA_DOCUMENTID_SQL, new Object[] {documentId});
    }

    /**
     * <p>
     * Update a record in DOCUMENT table.
     * </p>
     *
     * @param connection the connection to access database.
     * @param documentId the document id whose use count will be updated.
     * @param useCount the use count id to update.
     *
     * @return the row count affected by the operation.
     *
     * @throws SQLException if a database error occurs.
     */
    public static int updateDocumentUseCount(Connection connection, String documentId, int useCount)
        throws SQLException {
        return excuteUpdates(connection, UPDATE_DOCUMENT_USE_COUNT_SQL,
            new Object[] {new Integer(useCount), documentId});
    }

    /**
     * <p>
     * Closes JDBC resources.
     * </p>
     *
     * @param statement the SQL statement to be closed.
     */
    private static void closeResources(Statement statement) {
        if (statement != null) {
            try {
                statement.close();
            } catch (SQLException se) {
                // ignore
            }
        }
    }
}
