/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;
import com.topcoder.db.connectionfactory.DBConnectionFactoryImpl;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.IndexManager;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.MockWordSource;
import com.topcoder.document.index.persistence.UnitTestHelper;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSource;
import com.topcoder.document.index.wordsource.textfile.TextFileWordSource;

import junit.framework.TestCase;

import java.sql.Connection;

import java.util.HashSet;
import java.util.Locale;
import java.util.Set;


/**
 * <p>
 * Tests functionality and error cases of <code>SimpleAccessDBIndexPersistence</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class SimpleAccessDBIndexPersistenceUnitTest extends TestCase {
    /** Represents the delimiters used to separate words. */
    private static final String[] DELIMITERS = {" ", "\n", "\t", "\r", ",", "\"", ".", "!", "-", "?",
        "(", ")", "\0", "\\0", "\\\0", "\\\\0", "\\abcd\\c"};

    /** Represents the namespace to create the new <code>SimpleAccessDBIndexPersistence</code> instance for testing. */
    private static final String NAMESPACE =
        "com.topcoder.document.index.persistence.impl.db.SimpleAccessDBIndexPersistence";

    /** Represents the namespace to load DB connection factory configuration. */
    private static final String DB_NAMESPACE = "com.topcoder.document.index.persistence.impl.db";

    /** Represents the connection name to get DB connection factory configuration. */
    private static final String CONN_NAME = "SimpleAccessDBIndexPersistence";

    /** Represents the <code>SimpleAccessDBIndexPersistence</code> instance used for testing. */
    private SimpleAccessDBIndexPersistence persistence = null;

    /** Represents the database connection to access database. */
    private Connection connection;

    /** Represents a DB connection factory to create connections. */
    private DBConnectionFactory factory;

    /** Represents the <code>DocumentIndex</code> instance with small size of words used for testing. */
    private DocumentIndex smallDocumentIndex = null;

    /** Represents the <code>DocumentIndex</code> instance with large size of words used for testing. */
    private DocumentIndex largeDocumentIndex = null;

    /** Represents the <code>CollectionIndex</code> instance with small size of words used for testing. */
    private CollectionIndex smallCollectionIndex = null;

    /** Represents the <code>CollectionIndex</code> instance with large size of words used for testing. */
    private CollectionIndex largeCollectionIndex = null;

    /** Represents the <code>CollectionIndex</code> instance with two document index. */
    private CollectionIndex complexCollectionIndex = null;

    /**
     * <p>
     * Sets up the test environment. Valid configurations are loaded. The test instance is created. All the tables will
     * be cleared.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        UnitTestHelper.clearConfiguration();
        UnitTestHelper.addValidConfiguration();
        persistence = new SimpleAccessDBIndexPersistence(NAMESPACE);

        factory = new DBConnectionFactoryImpl(DB_NAMESPACE);
        connection = factory.createConnection(CONN_NAME);

        UnitTestHelper.clearSimpleDatabase(connection);

        // add the invalid config file
        UnitTestHelper.addConfigurationFile("InValidDBPersistenceConfig.xml");

        // Create the document index with small size of words
        WordSource doc = new MockWordSource(new String[] {
            "Microsoft", "word", "count", "Word", "This", "is", "a", "WORD", "what", "Yahoo", "Is", "this",
            "very", "good"}, new String[] {" ", ",", "?", "!"}, Locale.US);
        smallDocumentIndex = new DocumentIndex(doc);

        // Create the collection index with the above small document index
        smallCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {smallDocumentIndex},
                "collectionID1");

        // Create the document index with large size of words
        doc = new TextFileWordSource("test_files/Press.txt", DELIMITERS);
        largeDocumentIndex = new DocumentIndex(doc);

        // Create the collection index with the above large document index
        largeCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {largeDocumentIndex},
                "collectionID2");

        // Create the complex collection index with two document index
        complexCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {
            smallDocumentIndex, largeDocumentIndex}, "collectionID3");
    }

    /**
     * Removes the configuration if it exists. All the tables will be cleared.
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        UnitTestHelper.clearConfiguration();

        UnitTestHelper.clearSimpleDatabase(connection);

        connection.close();
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(String)</code> when the given namespace is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_NullNamespace() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(String)</code> when the given namespace is empty,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_EmptyNamespace() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence(" ");
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(String)</code> when the given namespace does not
     * exist, PersistenceConfigurationException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_NotExistNamespace() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence("NotExist");
            fail("PersistenceConfigurationException is expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(String)</code> when the property value of
     * ConnectionFactoryClassName is invalid, PersistenceConfigurationException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_NotExistClassNameProperyValue() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence("NotExistClassName");
            fail("PersistenceConfigurationException is expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(String)</code> when the property value of
     * ConnectionFactoryClassName is invalid, PersistenceConfigurationException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_NotCorrectClassNameProperyValue() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence("NotCorrectClassName");
            fail("PersistenceConfigurationException is expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(String)</code> when the property value of
     * ConnectionFactoryNamespace is invalid, PersistenceConfigurationException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_NotExistNamespaceProperyValue() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence("NotExistNamespace");
            fail("PersistenceConfigurationException is expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(String)</code> when the property value of
     * ConnectionFactoryNamespace is invalid, PersistenceConfigurationException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_NotCorrectNamespaceProperyValue() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence("NotCorrectNamespace");
            fail("PersistenceConfigurationException is expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(String)</code> when the property value of
     * ConnectionFactoryNamespace is missing, PersistenceConfigurationException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_MissingNamespaceProperyValue() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence("MissingNamespace");
            fail("PersistenceConfigurationException is expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>SimpleAccessDBIndexPersistence(String)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_StringAccuracy() throws Exception {
        assertTrue("The SimpleAccessDBIndexPersistence should extends AbstractDBIndexPersistence.",
            persistence instanceof AbstractDBIndexPersistence);
        assertEquals("The connectionName field should be correct.", CONN_NAME,
            UnitTestHelper.getPrivateField(persistence.getClass().getSuperclass(), persistence, "connectionName"));
        assertNotNull("The connectionFactory field should be set.",
            UnitTestHelper.getPrivateField(persistence.getClass().getSuperclass(), persistence, "connectionFactory"));
    }

    /**
     * <p>
     * Tests the constructor <code>SimpleAccessDBIndexPersistence(DBConnectionFactory, String)</code> when the given
     * connectionFactory is null, IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_NullConnectionFactory() throws Exception {
        try {
            new SimpleAccessDBIndexPersistence(null, CONN_NAME);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>SimpleAccessDBIndexPersistence(DBConnectionFactory, String)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testSimpleAccessDBIndexPersistence_DBConnectionFactoryStringAccuracy() throws Exception {
        persistence = new SimpleAccessDBIndexPersistence(this.factory, CONN_NAME);
        assertTrue("The SimpleAccessDBIndexPersistence should extends AbstractDBIndexPersistence.",
            persistence instanceof AbstractDBIndexPersistence);
        assertEquals("The connectionName field should be correct.", CONN_NAME,
            UnitTestHelper.getPrivateField(persistence.getClass().getSuperclass(), persistence, "connectionName"));
        assertEquals("The connectionFactory field should be set.", this.factory,
            UnitTestHelper.getPrivateField(persistence.getClass().getSuperclass(), persistence, "connectionFactory"));
    }

    /**
     * <p>
     * Tests the method <code>addDocumentIndex(DocumentIndex)</code> when the given documentIndex is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddDocumentIndex_NullDocumentIndex() throws Exception {
        try {
            persistence.addDocumentIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>addDocumentIndex(DocumentIndex)</code> when the given documentIndex exists in the
     * persistence, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddDocumentIndex_ExistsDocumentIndex() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        try {
            // add the index again
            persistence.addDocumentIndex(smallDocumentIndex);
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addDocumentIndex(DocumentIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddDocumentIndex_SmallSizeDocumentIndexAccuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        // validate the records in the database
        assertEquals("The count of records in WORD_POSITION table should be correct.", 14,
            UnitTestHelper.getRecordCount(connection, "WORD_POSITION"));
        assertEquals("The count of records in WORD table should be correct.", 10,
            UnitTestHelper.getRecordCount(connection, "WORD"));
        assertEquals("The count of records in DELIMITER table should be correct.", 4,
            UnitTestHelper.getRecordCount(connection, "DELIMITER"));
        assertEquals("The count of records in DOCUMENT_COLLECTION_DOCUMENT_XREF table should be correct.", 0,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION_DOCUMENT_XREF"));
        assertEquals("The count of records in DOCUMENT_COLLECTION table should be correct.", 0,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION"));
        assertEquals("The count of records in DOCUMENT table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT"));

        // validate the result
        DocumentIndex actual = persistence.getDocumentIndex(smallDocumentIndex.getWordSourceId());
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", smallDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", smallDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addDocumentIndex(DocumentIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddDocumentIndex_LargeSizeDocumentIndexAccuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(largeDocumentIndex);

        // validate the result
        DocumentIndex actual = persistence.getDocumentIndex(largeDocumentIndex.getWordSourceId());
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", largeDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", largeDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>getDocumentIndex(WordSourceId)</code> when the given wordSourceId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetDocumentIndex_NullWordSourceId() throws Exception {
        try {
            persistence.getDocumentIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getDocumentIndex(WordSourceId)</code> when the document index does not exist
     * in the persistence.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetDocumentIndex_NotExistDocumentIndexAccuracy() throws Exception {
        // get the index
        DocumentIndex actual = persistence.getDocumentIndex(smallDocumentIndex.getWordSourceId());

        // validate the result
        assertNull("The document index should be got properly.", actual);
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getDocumentIndex(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetDocumentIndex_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addDocumentIndex(largeDocumentIndex);

        // get the index
        DocumentIndex actual = persistence.getDocumentIndex(smallDocumentIndex.getWordSourceId());

        // validate the result
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", smallDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", smallDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));

        // get the index
        actual = persistence.getDocumentIndex(largeDocumentIndex.getWordSourceId());

        // validate the result
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", largeDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", largeDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>removeDocumentIndex(WordSourceId)</code> when the given wordSourceId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveDocumentIndex_NullWordSourceId() throws Exception {
        try {
            persistence.removeDocumentIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>removeDocumentIndex(WordSourceId)</code> when the given wordSourceId does not exist,
     * IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveDocumentIndex_NotExistDocumentIndex() throws Exception {
        try {
            persistence.removeDocumentIndex(smallDocumentIndex.getWordSourceId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>removeDocumentIndex(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveDocumentIndex_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addDocumentIndex(largeDocumentIndex);

        // remove the index
        persistence.removeDocumentIndex(largeDocumentIndex.getWordSourceId());

        // validate the result
        assertNull("The document id should be removed.",
            persistence.getDocumentIndex(largeDocumentIndex.getWordSourceId()));

        // validate the records in the database
        assertEquals("The count of records in WORD_POSITION table should be correct.", 14,
            UnitTestHelper.getRecordCount(connection, "WORD_POSITION"));
        assertEquals("The count of records in WORD table should be correct.", 10,
            UnitTestHelper.getRecordCount(connection, "WORD"));
        assertEquals("The count of records in DELIMITER table should be correct.", 4,
            UnitTestHelper.getRecordCount(connection, "DELIMITER"));
        assertEquals("The count of records in DOCUMENT_COLLECTION_DOCUMENT_XREF table should be correct.", 0,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION_DOCUMENT_XREF"));
        assertEquals("The count of records in DOCUMENT_COLLECTION table should be correct.", 0,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION"));
        assertEquals("The count of records in DOCUMENT table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT"));

        // validate the result of remaing document index
        DocumentIndex actual = persistence.getDocumentIndex(smallDocumentIndex.getWordSourceId());
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", smallDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", smallDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>addCollectionIndex(CollectionIndex)</code> when the given collectionIndex is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_NullCollectionIndex() throws Exception {
        try {
            persistence.addCollectionIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>addCollectionIndex(CollectionIndex)</code> when the given collectionIndex exists in the
     * persistence, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_ExistsCollectionIndex() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addCollectionIndex(smallCollectionIndex);

        try {
            // add the collection index again
            persistence.addCollectionIndex(smallCollectionIndex);
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_SmallSizeCollectionIndexAccuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addCollectionIndex(smallCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // validate the records in the database
        assertEquals("The count of records in WORD_POSITION table should be correct.", 14,
            UnitTestHelper.getRecordCount(connection, "WORD_POSITION"));
        assertEquals("The count of records in WORD table should be correct.", 10,
            UnitTestHelper.getRecordCount(connection, "WORD"));
        assertEquals("The count of records in DELIMITER table should be correct.", 4,
            UnitTestHelper.getRecordCount(connection, "DELIMITER"));
        assertEquals("The count of records in DOCUMENT_COLLECTION_DOCUMENT_XREF table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION_DOCUMENT_XREF"));
        assertEquals("The count of records in DOCUMENT_COLLECTION table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION"));
        assertEquals("The count of records in DOCUMENT table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT"));

        // Verify
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_LargeSizeCollectionIndexAccuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(largeCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(largeCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", largeCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", largeCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", largeCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_ComplexCollectionIndexAccuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(complexCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(complexCollectionIndex.getId());

        // validate the records in the database
        assertEquals("The count of records in DOCUMENT_COLLECTION_DOCUMENT_XREF table should be correct.", 2,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION_DOCUMENT_XREF"));
        assertEquals("The count of records in DOCUMENT_COLLECTION table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION"));
        assertEquals("The count of records in DOCUMENT table should be correct.", 2,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT"));

        // Verify
        assertEquals("The collection ID should be set.", complexCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", complexCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", complexCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>getCollectionIndex(String)</code> when the given collectionId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetCollectionIndex_NullCollectionId() throws Exception {
        try {
            persistence.getCollectionIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>getCollectionIndex(String)</code> when the given collectionId is empty,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetCollectionIndex_EmptyCollectionId() throws Exception {
        try {
            persistence.getCollectionIndex(" ");
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getCollectionIndex(String)</code> when the collection index does not exist in
     * the persistence.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetCollectionIndex_NotExistCollectionIndexAccuracy() throws Exception {
        // get the index
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // validate the result
        assertNull("The collection index should be got properly.", actual);
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getCollectionIndex(String)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetCollectionIndex_Accuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addCollectionIndex(smallCollectionIndex);
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(largeCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));

        // get the collection index
        actual = persistence.getCollectionIndex(largeCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", largeCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", largeCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", largeCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>removeCollectionIndex(String)</code> when the given collectionId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveCollectionIndex_NullCollectionId() throws Exception {
        try {
            persistence.removeCollectionIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>removeCollectionIndex(String)</code> when the given collectionId is empty,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveCollectionIndex_EmptyCollectionId() throws Exception {
        try {
            persistence.removeCollectionIndex(" ");
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>removeCollectionIndex(String)</code> when the given collection index does not exist,
     * IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveCollectionIndex_NotExistCollectionIndex() throws Exception {
        try {
            persistence.removeCollectionIndex(smallCollectionIndex.getId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>removeCollectionIndex(String)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveCollectionIndex_Accuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addCollectionIndex(smallCollectionIndex);
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(largeCollectionIndex);

        // remove the collection index
        persistence.removeCollectionIndex(largeCollectionIndex.getId());
        assertNull("The collection index should be removed from the persistence.",
            persistence.getCollectionIndex(largeCollectionIndex.getId()));

        // validate the records in the database
        assertEquals("The count of records in DOCUMENT_COLLECTION_DOCUMENT_XREF table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION_DOCUMENT_XREF"));
        assertEquals("The count of records in DOCUMENT_COLLECTION table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION"));

        // validate the remaining result
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>updateCollectionIndex(CollectionIndex)</code> when the given collectionIndex is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testUpdateCollectionIndex_NullCollectionIndex() throws Exception {
        try {
            persistence.updateCollectionIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>updateCollectionIndex(CollectionIndex)</code> when the given collection index does not
     * exist, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testUpdateCollectionIndex_NotExistCollectionIndex() throws Exception {
        try {
            persistence.updateCollectionIndex(smallCollectionIndex);
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>updateCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testUpdateCollectionIndex_Accuracy() throws Exception {
        // add the empty index
        IndexManager manager = new IndexManager(persistence);
        persistence.addCollectionIndex(new CollectionIndex(manager, smallCollectionIndex.getId()));

        // update the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.updateCollectionIndex(smallCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The count of records in DOCUMENT_COLLECTION_DOCUMENT_XREF table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION_DOCUMENT_XREF"));
        assertEquals("The count of records in DOCUMENT_COLLECTION table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION"));
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());

        // change the collection index and update it
        smallCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {largeDocumentIndex},
                smallCollectionIndex.getId());
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.updateCollectionIndex(smallCollectionIndex);

        // get the collection index
        actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The count of records in DOCUMENT_COLLECTION_DOCUMENT_XREF table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION_DOCUMENT_XREF"));
        assertEquals("The count of records in DOCUMENT_COLLECTION table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION"));
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());

        // change the collection index and update it
        smallCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {
            smallDocumentIndex, largeDocumentIndex}, smallCollectionIndex.getId());
        persistence.updateCollectionIndex(smallCollectionIndex);

        // get the collection index
        actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The count of records in DOCUMENT_COLLECTION_DOCUMENT_XREF table should be correct.", 2,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION_DOCUMENT_XREF"));
        assertEquals("The count of records in DOCUMENT_COLLECTION table should be correct.", 1,
            UnitTestHelper.getRecordCount(connection, "DOCUMENT_COLLECTION"));
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());
    }

    /**
     * <p>
     * Tests the method <code>increaseDocumentUseCount(WordSourceId)</code> when the given wordSourceId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_NullWordSourceId() throws Exception {
        try {
            persistence.increaseDocumentUseCount(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>increaseDocumentUseCount(WordSourceId)</code> when the given document index does not
     * exist, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_NotExistDocumentIndex() throws Exception {
        try {
            persistence.increaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>increaseDocumentUseCount(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        // increase the use count
        persistence.increaseDocumentUseCount(smallDocumentIndex.getWordSourceId());

        // check
        assertEquals("The use count should be correct.", 1,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>decreaseDocumentUseCount(WordSourceId)</code> when the given wordSourceId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_NullWordSourceId() throws Exception {
        try {
            persistence.decreaseDocumentUseCount(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>decreaseDocumentUseCount(WordSourceId)</code> when the given document index does not
     * exist, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_NotExistDocumentIndex() throws Exception {
        try {
            persistence.decreaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>decreaseDocumentUseCount(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        // increase the use count
        persistence.increaseDocumentUseCount(smallDocumentIndex.getWordSourceId());

        // decrease the use count
        persistence.decreaseDocumentUseCount(smallDocumentIndex.getWordSourceId());

        // check
        assertEquals("The use count should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>decreaseDocumentUseCount(WordSourceId)</code>. Can not decrease the use count
     * value since it is already 0, so IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_AlreadyZeroAccuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        try {
            persistence.decreaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getIndexedDocuments()</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetIndexedDocuments_Accuracy() throws Exception {
        // add the document index into persistence
        Set expected = new HashSet();
        persistence.addDocumentIndex(smallDocumentIndex);
        expected.add(smallDocumentIndex.getWordSourceId());
        persistence.addDocumentIndex(largeDocumentIndex);
        expected.add(largeDocumentIndex.getWordSourceId());

        // Verify
        assertEquals("The document ID set should be got properly.", expected, persistence.getIndexedDocuments());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getDocumentUseCount(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetDocumentUseCount_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        // Verify
        assertEquals("Use count should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
        persistence.increaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
        assertEquals("Use count should be correct.", 1,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
        persistence.decreaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
        assertEquals("Use count should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }
}
