/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.IndexManager;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.MockWordSource;
import com.topcoder.document.index.persistence.UnitTestHelper;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSource;
import com.topcoder.document.index.wordsource.textfile.TextFileWordSource;

import junit.framework.TestCase;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import java.util.HashSet;
import java.util.Locale;
import java.util.Set;


/**
 * <p>
 * Tests functionality and error cases of <code>XmlIndexPersistence</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class XmlIndexPersistenceUnitTest extends TestCase {
    /**
     * <p>
     * This represent the file name used to represent a document usage as defined by the persistence contract. There is
     * only one such file per xml persistence name space.
     * </p>
     */
    private static final String DOC_USAGE_FILE_NAME = "document_usage.xml";

    /** Represents the delimiters used to separate words. */
    private static final String[] DELIMITERS = {" ", "\n", "\t", "\r", ",", "\"", ".", "!", "-", "?",
        "(", ")", "\0", "\\0", "\\\0", "\\\\0", "\\abcd\\c"};

    /** Represents the namespace to create the new <code>XmlIndexPersistence</code> instance for testing. */
    private static final String NAMESPACE = "com.topcoder.document.index.persistence.impl.xml.XmlIndexPersistence";

    /** Represents the directory used to store all the xml persistence files used for testing. */
    private static final String PERSISTENCE_DIR = "test_files/XmlPersistence";

    /** Represents the <code>XmlIndexPersistence</code> instance used for testing. */
    private XmlIndexPersistence persistence = null;

    /** Represents the <code>DocumentIndex</code> instance with small size of words used for testing. */
    private DocumentIndex smallDocumentIndex = null;

    /** Represents the <code>DocumentIndex</code> instance with large size of words used for testing. */
    private DocumentIndex largeDocumentIndex = null;

    /** Represents the <code>CollectionIndex</code> instance with small size of words used for testing. */
    private CollectionIndex smallCollectionIndex = null;

    /** Represents the <code>CollectionIndex</code> instance with large size of words used for testing. */
    private CollectionIndex largeCollectionIndex = null;

    /** Represents the <code>CollectionIndex</code> instance with two document index. */
    private CollectionIndex complexCollectionIndex = null;

    /**
     * <p>
     * Sets up the test environment. Valid configurations are loaded. The test instance is created.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        UnitTestHelper.clearConfiguration();
        UnitTestHelper.addValidConfiguration();
        persistence = new XmlIndexPersistence(NAMESPACE);

        // clear all files
        File[] files = new File(PERSISTENCE_DIR).listFiles();

        for (int i = 0; i < files.length; i++) {
            files[i].delete();
        }

        // create the document_usage.xml for testing.
        File file = new File(PERSISTENCE_DIR, DOC_USAGE_FILE_NAME);
        BufferedWriter writer = new BufferedWriter(new FileWriter(file));
        writer.write("<documents></documents>");
        writer.flush();
        writer.close();

        // add the invalid config file
        UnitTestHelper.addConfigurationFile("InValidXmlPersistenceConfig.xml");

        // Create the document index with small size of words
        WordSource doc = new MockWordSource(new String[] {
            "Microsoft", "word", "count", "Word", "This", "is", "a", "WORD", "what", "Yahoo", "Is", "this",
            "very", "good"}, new String[] {" ", ",", "?", "!"}, Locale.US);
        smallDocumentIndex = new DocumentIndex(doc);

        // Create the collection index with the above small document index
        smallCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {smallDocumentIndex},
                "collectionID1");

        // Create the document index with large size of words
        doc = new TextFileWordSource("test_files/Press.txt", DELIMITERS);
        largeDocumentIndex = new DocumentIndex(doc);

        // Create the collection index with the above large document index
        largeCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {largeDocumentIndex},
                "collectionID2");

        // Create the complex collection index with two document index
        complexCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {
            smallDocumentIndex, largeDocumentIndex}, "collectionID3");
    }

    /**
     * Removes the configuration if it exists.
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        UnitTestHelper.clearConfiguration();

        // clear all files
        File[] files = new File(PERSISTENCE_DIR).listFiles();

        for (int i = 0; i < files.length; i++) {
            files[i].delete();
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given namespace is <code>null</code>. Expect
     * IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NullNamespace() throws Exception {
        try {
            new XmlIndexPersistence(null);
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given namespace is empty. Expect
     * IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_EmptyNamespace() throws Exception {
        try {
            new XmlIndexPersistence(" ");
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given namespace does not exist. Expect
     * PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NotExistNamespace() throws Exception {
        try {
            new XmlIndexPersistence("NotExist");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeLimit property value is
     * missing. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_MissingFileSizeLimitPropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("MissingFileSizeLimit");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeLimit property value is not
     * valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NotIntFileSizeLimitPropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("NotIntFileSizeLimit");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeLimit property value is not
     * valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NegativeFileSizeLimitPropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("NegativeFileSizeLimit");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeLimit property value is not
     * valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_TooLargeFileSizeLimitPropertyValue1() throws Exception {
        try {
            new XmlIndexPersistence("TooLargeFileSizeLimit1");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeLimit property value is not
     * valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_TooLargeFileSizeLimitPropertyValue2() throws Exception {
        try {
            new XmlIndexPersistence("TooLargeFileSizeLimit2");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeThresholdValue property value
     * is missing. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_MissingFileSizeThresholdValuePropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("MissingFileSizeThresholdValue");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeThresholdValue property value
     * is not valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NotIntFileSizeThresholdValuePropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("NotIntFileSizeThresholdValue");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeThresholdValue property value
     * is not valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NegativeFileSizeThresholdValuePropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("NegativeFileSizeThresholdValue");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeThresholdValue property value
     * is not valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_TooLargeFileSizeThresholdValuePropertyValue1() throws Exception {
        try {
            new XmlIndexPersistence("TooLargeFileSizeThresholdValue1");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given fileSizeThresholdValue property value
     * is not valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_TooLargeFileSizeThresholdValuePropertyValue2() throws Exception {
        try {
            new XmlIndexPersistence("TooLargeFileSizeThresholdValue2");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given persistenceDirPath property value is
     * missing. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_MissingPersistenceDirPathPropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("MissingPersistenceDirPath");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given persistenceDirPath property value is
     * empty. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_EmptyPersistenceDirPathPropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("EmptyPersistenceDirPath");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given persistenceDirPath property value is
     * not valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NotExistPersistenceDirPathPropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("NotExistPersistenceDirPath");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(String)</code> when the given persistenceDirPath property value is
     * not valid. Expect PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NotDirectoryPersistenceDirPathPropertyValue() throws Exception {
        try {
            new XmlIndexPersistence("NotDirectoryPersistenceDirPath");
            fail("Should throw PersistenceConfigurationException.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>XmlIndexPersistence(String)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_StringAccuracy() throws Exception {
        assertTrue("The XmlIndexPersistence should extend IndexPersistence.",
                persistence instanceof XmlIndexPersistence);
        assertEquals("The fileSizeLimit field should be correct.", "" + (2000 * 1024 * 1024),
            UnitTestHelper.getPrivateField(persistence.getClass(), persistence, "fileSizeLimit").toString());
        assertEquals("The fileSizeThreshold field should be correct.", "" + (10 * 1024),
            UnitTestHelper.getPrivateField(persistence.getClass(), persistence, "fileSizeThreshold").toString());
        assertEquals("The persistenceDirPath field should be correct.", PERSISTENCE_DIR,
            UnitTestHelper.getPrivateField(persistence.getClass(), persistence, "persistenceDirPath"));
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(int, int, String)</code> when the given fileSizeLimit value is not
     * valid. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NegativeFileSizeLimit() throws Exception {
        try {
            new XmlIndexPersistence(-1, 1, PERSISTENCE_DIR);
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(int, int, String)</code> when the given fileSizeLimit value is not
     * valid. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_TooLargeFileSizeLimit() throws Exception {
        try {
            new XmlIndexPersistence(10000000, 1, PERSISTENCE_DIR);
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(int, int, String)</code> when the given fileSizeThresholdValue value
     * is not valid. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NegativeFileSizeThresholdValue() throws Exception {
        try {
            new XmlIndexPersistence(1, -1, PERSISTENCE_DIR);
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>IllegalArgumentException(int, int, String)</code> when the given fileSizeThresholdValue
     * value is not valid. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_TooLargeFileSizeThresholdValue() throws Exception {
        try {
            new XmlIndexPersistence(1, 100000000, PERSISTENCE_DIR);
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(int, int, String)</code> when the given persistenceDirPath value is
     * null. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NullPersistenceDirPath() throws Exception {
        try {
            new XmlIndexPersistence(1, 1, null);
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(int, int, String)</code> when the given persistenceDirPath value is
     * empty. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_EmptyPersistenceDirPath() throws Exception {
        try {
            new XmlIndexPersistence(1, 1, " ");
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(int, int, String)</code> when the given persistenceDirPath value is
     * not valid. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NotExistPersistenceDirPath() throws Exception {
        try {
            new XmlIndexPersistence(1, 1, "notExist");
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlIndexPersistence(int, int, String)</code> when the given persistenceDirPath value is
     * not valid. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_NotDirectoryPersistenceDirPath() throws Exception {
        try {
            new XmlIndexPersistence(1, 1, "test_files/XmlPersistence/document_usage.xml");
            fail("Should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>XmlIndexPersistence(int, int, String)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlIndexPersistence_IntIntStringAccuracy() throws Exception {
        persistence = new XmlIndexPersistence(1, 1, PERSISTENCE_DIR);
        assertTrue("The XmlIndexPersistence should extend IndexPersistence.",
                persistence instanceof XmlIndexPersistence);
        assertEquals("The fileSizeLimit field should be correct.", "" + (1 * 1024 * 1024),
            UnitTestHelper.getPrivateField(persistence.getClass(), persistence, "fileSizeLimit").toString());
        assertEquals("The fileSizeThreshold field should be correct.", "" + (1 * 1024),
            UnitTestHelper.getPrivateField(persistence.getClass(), persistence, "fileSizeThreshold").toString());
        assertEquals("The persistenceDirPath field should be correct.", PERSISTENCE_DIR,
            UnitTestHelper.getPrivateField(persistence.getClass(), persistence, "persistenceDirPath"));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addDocumentIndex(DocumentIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddDocumentIndex_SmallSizeDocumentIndexAccuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        // validate the result
        DocumentIndex actual = persistence.getDocumentIndex(smallDocumentIndex.getWordSourceId());
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", smallDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", smallDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addDocumentIndex(DocumentIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddDocumentIndex_LargeSizeDocumentIndexAccuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(largeDocumentIndex);

        // validate the result
        DocumentIndex actual = persistence.getDocumentIndex(largeDocumentIndex.getWordSourceId());
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", largeDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());

        assertEquals("The word map should be correct.", largeDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addDocumentIndex(DocumentIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddDocumentIndex_MutliPageDocumentIndexAccuracy() throws Exception {
        // add the index
        persistence = new XmlIndexPersistence(1, 999, PERSISTENCE_DIR);
        persistence.addDocumentIndex(largeDocumentIndex);

        // validate the result
        DocumentIndex actual = persistence.getDocumentIndex(largeDocumentIndex.getWordSourceId());
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", largeDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());

        assertEquals("The word map should be correct.", largeDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>getDocumentIndex(WordSourceId)</code> when the given wordSourceId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetDocumentIndex_NullWordSourceId() throws Exception {
        try {
            persistence.getDocumentIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getDocumentIndex(WordSourceId)</code> when the document index does not exist
     * in the persistence.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetDocumentIndex_NotExistDocumentIndexAccuracy() throws Exception {
        // get the index
        DocumentIndex actual = persistence.getDocumentIndex(smallDocumentIndex.getWordSourceId());

        // validate the result
        assertNull("The document index should be got properly.", actual);
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getDocumentIndex(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetDocumentIndex_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addDocumentIndex(largeDocumentIndex);

        // get the index
        DocumentIndex actual = persistence.getDocumentIndex(smallDocumentIndex.getWordSourceId());

        // validate the result
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", smallDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", smallDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));

        // get the index
        actual = persistence.getDocumentIndex(largeDocumentIndex.getWordSourceId());

        // validate the result
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", largeDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", largeDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>removeDocumentIndex(WordSourceId)</code> when the given wordSourceId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveDocumentIndex_NullWordSourceId() throws Exception {
        try {
            persistence.removeDocumentIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>removeDocumentIndex(WordSourceId)</code> when the given wordSourceId does not exist,
     * IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveDocumentIndex_NotExistDocumentIndex() throws Exception {
        try {
            persistence.removeDocumentIndex(smallDocumentIndex.getWordSourceId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>removeDocumentIndex(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveDocumentIndex_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addDocumentIndex(largeDocumentIndex);

        // remove the index
        persistence.removeDocumentIndex(largeDocumentIndex.getWordSourceId());

        // validate the result
        assertNull("The document id should be removed.",
            persistence.getDocumentIndex(largeDocumentIndex.getWordSourceId()));

        // validate the result of remaing document index
        DocumentIndex actual = persistence.getDocumentIndex(smallDocumentIndex.getWordSourceId());
        assertNotNull("The document index should be got properly.", actual);
        assertEquals("The document ID should be correct.", smallDocumentIndex.getWordSourceId(),
            actual.getWordSourceId());
        assertEquals("The word map should be correct.", smallDocumentIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>addCollectionIndex(CollectionIndex)</code> when the given collectionIndex is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_NullCollectionIndex() throws Exception {
        try {
            persistence.addCollectionIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>addCollectionIndex(CollectionIndex)</code> when the given collectionIndex exists in the
     * persistence, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_ExistsCollectionIndex() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addCollectionIndex(smallCollectionIndex);

        try {
            // add the collection index again
            persistence.addCollectionIndex(smallCollectionIndex);
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_SmallSizeCollectionIndexAccuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addCollectionIndex(smallCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_LargeSizeCollectionIndexAccuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(largeCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(largeCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", largeCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", largeCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", largeCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_ComplexCollectionIndexAccuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(complexCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(complexCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", complexCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", complexCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", complexCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>addCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAddCollectionIndex_MultiPageCollectionIndexAccuracy() throws Exception {
        persistence = new XmlIndexPersistence(1, 999, PERSISTENCE_DIR);

        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(largeCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(largeCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", largeCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", largeCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", largeCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>getCollectionIndex(String)</code> when the given collectionId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetCollectionIndex_NullCollectionId() throws Exception {
        try {
            persistence.getCollectionIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>getCollectionIndex(String)</code> when the given collectionId is empty,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetCollectionIndex_EmptyCollectionId() throws Exception {
        try {
            persistence.getCollectionIndex(" ");
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getCollectionIndex(String)</code> when the collection index does not exist in
     * the persistence.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetCollectionIndex_NotExistCollectionIndexAccuracy() throws Exception {
        // get the index
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // validate the result
        assertNull("The collection index should be got properly.", actual);
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getCollectionIndex(String)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetCollectionIndex_Accuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addCollectionIndex(smallCollectionIndex);
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(largeCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));

        // get the collection index
        actual = persistence.getCollectionIndex(largeCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", largeCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", largeCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", largeCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(largeDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>removeCollectionIndex(String)</code> when the given collectionId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveCollectionIndex_NullCollectionId() throws Exception {
        try {
            persistence.removeCollectionIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>removeCollectionIndex(String)</code> when the given collectionId is empty,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveCollectionIndex_EmptyCollectionId() throws Exception {
        try {
            persistence.removeCollectionIndex(" ");
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>removeCollectionIndex(String)</code> when the given collection index does not exist,
     * IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveCollectionIndex_NotExistCollectionIndex() throws Exception {
        try {
            persistence.removeCollectionIndex(smallCollectionIndex.getId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>removeCollectionIndex(String)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRemoveCollectionIndex_Accuracy() throws Exception {
        // add the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.addCollectionIndex(smallCollectionIndex);
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.addCollectionIndex(largeCollectionIndex);

        // remove the collection index
        persistence.removeCollectionIndex(largeCollectionIndex.getId());
        assertNull("The collection index should be removed from the persistence.",
            persistence.getCollectionIndex(largeCollectionIndex.getId()));

        // validate the remaining result
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());
        assertEquals("The use count value should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>updateCollectionIndex(CollectionIndex)</code> when the given collectionIndex is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testUpdateCollectionIndex_NullCollectionIndex() throws Exception {
        try {
            persistence.updateCollectionIndex(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>updateCollectionIndex(CollectionIndex)</code> when the given collection index does not
     * exist, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testUpdateCollectionIndex_NotExistCollectionIndex() throws Exception {
        try {
            persistence.updateCollectionIndex(smallCollectionIndex);
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>updateCollectionIndex(CollectionIndex)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testUpdateCollectionIndex_Accuracy() throws Exception {
        // add the empty index
        IndexManager manager = new IndexManager(persistence);
        persistence.addCollectionIndex(new CollectionIndex(manager, smallCollectionIndex.getId()));

        // update the collection index, should first add the document index into persistence
        persistence.addDocumentIndex(smallDocumentIndex);
        persistence.updateCollectionIndex(smallCollectionIndex);

        // get the collection index
        CollectionIndex actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());

        // change the collection index and update it
        smallCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {largeDocumentIndex},
                smallCollectionIndex.getId());
        persistence.addDocumentIndex(largeDocumentIndex);
        persistence.updateCollectionIndex(smallCollectionIndex);

        // get the collection index
        actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());

        // change the collection index and update it
        smallCollectionIndex = UnitTestHelper.createCollectionIndex(new DocumentIndex[] {
            smallDocumentIndex, largeDocumentIndex}, smallCollectionIndex.getId());
        persistence.updateCollectionIndex(smallCollectionIndex);

        // get the collection index
        actual = persistence.getCollectionIndex(smallCollectionIndex.getId());

        // Verify
        assertEquals("The collection ID should be set.", smallCollectionIndex.getId(), actual.getId());
        assertEquals("The document ID set should be set.", smallCollectionIndex.getAllDocumentIds(),
            actual.getAllDocumentIds());
        assertEquals("The word map should be set.", smallCollectionIndex.getWords(), actual.getWords());
    }

    /**
     * <p>
     * Tests the method <code>increaseDocumentUseCount(WordSourceId)</code> when the given wordSourceId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_NullWordSourceId() throws Exception {
        try {
            persistence.increaseDocumentUseCount(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>increaseDocumentUseCount(WordSourceId)</code> when the given document index does not
     * exist, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_NotExistDocumentIndex() throws Exception {
        try {
            persistence.increaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>increaseDocumentUseCount(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        // increase the use count
        persistence.increaseDocumentUseCount(smallDocumentIndex.getWordSourceId());

        // check
        assertEquals("The use count should be correct.", 1,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the method <code>decreaseDocumentUseCount(WordSourceId)</code> when the given wordSourceId is null,
     * IllegalArgumentException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_NullWordSourceId() throws Exception {
        try {
            persistence.decreaseDocumentUseCount(null);
            fail("IllegalArgumentException is expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>decreaseDocumentUseCount(WordSourceId)</code> when the given document index does not
     * exist, IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_NotExistDocumentIndex() throws Exception {
        try {
            persistence.decreaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>decreaseDocumentUseCount(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        // increase the use count
        persistence.increaseDocumentUseCount(smallDocumentIndex.getWordSourceId());

        // decrease the use count
        persistence.decreaseDocumentUseCount(smallDocumentIndex.getWordSourceId());

        // check
        assertEquals("The use count should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>decreaseDocumentUseCount(WordSourceId)</code>. Can not decrease the use count
     * value since it is already 0, so IndexPersistenceException is expected.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_AlreadyZeroAccuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        try {
            persistence.decreaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
            fail("IndexPersistenceException is expected.");
        } catch (IndexPersistenceException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getIndexedDocuments()</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetIndexedDocuments_Accuracy() throws Exception {
        // add the document index into persistence
        Set expected = new HashSet();
        persistence.addDocumentIndex(smallDocumentIndex);
        expected.add(smallDocumentIndex.getWordSourceId());
        persistence.addDocumentIndex(largeDocumentIndex);
        expected.add(largeDocumentIndex.getWordSourceId());

        // Verify
        assertEquals("The document ID set should be got properly.", expected, persistence.getIndexedDocuments());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getDocumentUseCount(WordSourceId)</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGetDocumentUseCount_Accuracy() throws Exception {
        // add the index
        persistence.addDocumentIndex(smallDocumentIndex);

        // Verify
        assertEquals("Use count should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
        persistence.increaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
        assertEquals("Use count should be correct.", 1,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
        persistence.decreaseDocumentUseCount(smallDocumentIndex.getWordSourceId());
        assertEquals("Use count should be correct.", 0,
            persistence.getDocumentUseCount(smallDocumentIndex.getWordSourceId()));
    }
}
